import { ElementType, useState } from "react";
import { createSearchParams, useNavigate } from "react-router-dom";
import { ToggleType } from "../WithHocToggle";

export const WithHocSearch = (Component: ElementType) => {
  return function WithHocSearchComponent({ ...props }: {
    [key: string]: any;
  }) {
    const [search, setSearch] = useState<string>('');
    const [searchToggle, setSearchToggle] = useState<boolean>(false);
    const searchNavigate = useNavigate();

    return (
      <Component
        handleSearch={({ target }: any) => setSearch(target.value)}
        searchToggle={(
          onTrue: string = 'show',
          onFalse: string = '',
          extraCondition: any = null
        ) => (extraCondition) || searchToggle ? onTrue : onFalse}
        handleSearchToggle={() => setSearchToggle(!searchToggle)}
        handleSearchBlur={() => setSearchToggle(false)}
        handleSearchTrigger={(e: any, path?: string) => {
          e.preventDefault();
          setSearch(e.currentTarget.elements[0].value);

          if (path) {
            searchNavigate({
              pathname: path,
              search: createSearchParams({
                search: search,
              }).toString(),
            });
          }
        }}
        handleSearchReset={({ keyCode }: any) => {
          if (keyCode === 8) setSearch('');
        }}
        filterData={(data: any[]) => {
          return data.filter(datum => {
            let searchableDatum: string[] = [];

            Object.keys(datum).slice(1).forEach(attribute => {
              searchableDatum.push(String(datum[attribute]));
            });

            return searchableDatum.join(' ').toLowerCase()
              .indexOf(search.toLowerCase()) >= 0;
          });
        }}
        {...props}
      />
    );
  };
};

export type HandleSearchType = ({ target }: any) => void;
export type SearchToggleType = ToggleType;
export type HandleSearchToggleType = () => void;
export type HandleSearchBlurType = HandleSearchToggleType;
export type HandleSearchTriggerType = (e: any) => void;
export type HandleSearchResetType = ({ keyCode }: any) => void;
export type FilterDataType = (data: any[]) => any[];
